@@ -8,7 +8,7 @@ from .tree import DWARFTreeModel, has_code_location, cu_sort_key
 from .scriptdlg import ScriptDlg
 from .ui import setup_ui
 
-version = (2, 10)
+version = (2, 11)
 
 # TODO:
 # On MacOS, start without a main window, instead show the Open dialog
@@ -263,17 +263,18 @@ class TheWindow(QMainWindow):
     # For both back and forward, delta=1 for back, -1 for forward
     # Checked because back-forward buttons can't be disabled
     def on_nav(self, delta):
-        np = self.navpos + delta
-        if np < 0 or np >= len(self.navhistory):
-            return
-        self.navpos = np
-        navitem = self.navhistory[np]
-        tree_index = self.tree_model.index_for_navitem(navitem)
-        self.in_tree_nav = True
-        self.the_tree.setCurrentIndex(tree_index) # Causes on_tree_selection internally
-        self.in_tree_nav = False
-        self.back_menuitem.setEnabled(np < len(self.navhistory) - 1)
-        self.forward_menuitem.setEnabled(np > 0)
+        if self.tree_model: # Maybe fix for #1461? Short out nav if no file loaded
+            np = self.navpos + delta
+            if np < 0 or np >= len(self.navhistory):
+                return
+            self.navpos = np
+            navitem = self.navhistory[np]
+            tree_index = self.tree_model.index_for_navitem(navitem)
+            self.in_tree_nav = True
+            self.the_tree.setCurrentIndex(tree_index) # Causes on_tree_selection internally
+            self.in_tree_nav = False
+            self.back_menuitem.setEnabled(np < len(self.navhistory) - 1)
+            self.forward_menuitem.setEnabled(np > 0)
 
     def followref(self, index = None):
         self.start_wait() # TODO: only show the wait cursor if it's indeed time consuming
@@ -485,13 +486,13 @@ class TheWindow(QMainWindow):
     def on_copyvalue(self):
         t = self.details_table if self.details_table.hasFocus() else self.die_table
         m = t.model()
-        self.on_copy(m.data(t.currentIndex(), Qt.ItemDataRole.DisplayRole))
+        self.on_copy(m.data(t.currentIndex(), Qt.ItemDataRole.DisplayRole) or "")
 
     def on_copyline(self):
         t = self.details_table if self.details_table.hasFocus() else self.die_table
         m = t.model()
         row = t.currentIndex().row()
-        line = "\t".join(m.data(m.index(row, c, QModelIndex()), Qt.ItemDataRole.DisplayRole)
+        line = "\t".join(m.data(m.index(row, c, QModelIndex()), Qt.ItemDataRole.DisplayRole) or ""
             for c in range(0, m.columnCount(QModelIndex())))
         self.on_copy(line)
 
@@ -499,7 +500,7 @@ class TheWindow(QMainWindow):
         t = self.details_table if self.details_table.hasFocus() else self.die_table
         m = t.model()
         table_text = "\n".join(
-                "\t".join(m.data(m.index(r, c, QModelIndex()), Qt.ItemDataRole.DisplayRole)
+                "\t".join(m.data(m.index(r, c, QModelIndex()), Qt.ItemDataRole.DisplayRole)  or ""
                 for c in range(0, m.columnCount(QModelIndex())))
             for r in range(0, m.rowCount(QModelIndex())))
         self.on_copy(table_text)
@@ -555,6 +556,9 @@ def main():
         on_exception.prev_exchook = sys.excepthook
         sys.excepthook = on_exception
 
+    from .patch import monkeypatch
+    monkeypatch()
+
     TheApp().start()
             
 
