@@ -4,6 +4,7 @@ from PyQt6.QtGui import QBrush, QFont
 from elftools.dwarf.locationlists import LocationParser, LocationExpr
 from elftools.dwarf.dwarf_expr import DWARFExprParser, DWARFExprOp, DW_OP_opcode2name
 from elftools.dwarf.descriptions import _DESCR_DW_LANG, _DESCR_DW_ATE, _DESCR_DW_ACCESS, _DESCR_DW_INL, _REG_NAMES_x86, _REG_NAMES_x64
+from elftools.common.exceptions import ELFParseError
 from .dwarfone import DWARFExprParserV1
 
 MAX_INLINE_BYTEARRAY_LEN = 32
@@ -65,8 +66,12 @@ _fixed_font = None
 _ll_headers = ("Attribute", "Offset", "Form", "Raw", "Value")
 _noll_headers = ("Attribute", "Form", "Value")
 _meta_desc = ('DIE offset', 'DIE size', 'Abbrev code', 'Has children') # Anything else?
-_meta_count = 4
+_meta_count = 4 # Extra rows if low level detail showing is set
 
+class NoBaseError(Exception):
+    pass
+
+#May return None!
 def get_cu_base(die):
     top_die = die.cu.get_top_DIE()
     if 'DW_AT_low_pc' in top_die.attributes:
@@ -75,7 +80,7 @@ def get_cu_base(die):
         return top_die.attributes['DW_AT_entry_pc'].value
     # TODO: ranges?
     else:
-        raise ValueError("Can't find the base address for the location list")
+        raise NoBaseError()
 
 def is_int_list(val):
     return isinstance(val, list) and len(val) > 0 and isinstance(val[0], int)
@@ -84,6 +89,9 @@ def is_long_blob(attr):
     val = attr.value
     return ((isinstance(val, bytes) and attr.form not in ('DW_FORM_strp', 'DW_FORM_string')) or is_int_list(val)) and len(val) > MAX_INLINE_BYTEARRAY_LEN
 
+def is_block(form):
+    return form in ('DW_FORM_block', 'DW_FORM_block1', 'DW_FORM_block2', 'DW_FORM_block4')
+
 class DIETableModel(QAbstractTableModel):
     def __init__(self, die, prefix, lowlevel, hex, regnames):
         QAbstractTableModel.__init__(self)
@@ -120,6 +128,10 @@ class DIETableModel(QAbstractTableModel):
         return self.attr_data(index, role) if row >= self.meta_count else self.meta_data(index, role)
 
     def attr_data(self, index, role):
+        irow = index.row()
+        meta_count = self.meta_count
+        self_keys_len = len(self.keys)
+        self_die_keys_len = len(self.die.attributes.keys())
         row = index.row() - self.meta_count
         key = self.keys[row]
         attr = self.attributes[key]
@@ -224,58 +236,74 @@ class DIETableModel(QAbstractTableModel):
 
     # Big DIE attribute value interpreter
     def format_value(self, attr):
-        key = attr.name
-        val = attr.value
-        form = attr.form
-        if form == 'DW_FORM_addr' and isinstance(val, int):
-            return hex(val)
-        elif form == 'DW_FORM_flag_present':
-            return ''
-        elif form in ('DW_FORM_ref0', 'DW_FORM_ref1', 'DW_FORM_ref2', 'DW_FORM_ref4', 'DW_FORM_ref8', 'DW_FORM_ref_addr'):
-            return "Ref: 0x%x" % val # There are several other reference forms in the spec
-        elif LocationParser.attribute_has_location(attr, self.die.cu['version']):
-            ll = self.parse_location(attr)
-            if isinstance(ll, LocationExpr):
-                return '; '.join(self.dump_expr(ll.loc_expr))
-            else:
-                return "Loc list: 0x%x" % attr.value
-        elif key == 'DW_AT_language':
-            return "%d %s" % (val, _DESCR_DW_LANG[val]) if val in _DESCR_DW_LANG else val
-        elif key == 'DW_AT_encoding':
-            return "%d %s" % (val, _DESCR_DW_ATE[val]) if val in _DESCR_DW_ATE else val
-        elif key == 'DW_AT_accessibility':
-            return "%d %s" % (val, _DESCR_DW_ACCESS[val]) if val in _DESCR_DW_ACCESS else val
-        elif key == 'DW_AT_inline':
-            return "%d %s" % (val, _DESCR_DW_INL[val]) if val in _DESCR_DW_INL else val
-        elif key in ('DW_AT_decl_file', 'DW_AT_call_file'):
+        try:
+            die = self.die
             cu = self.die.cu
-            if cu._lineprogram is None:
-                cu._lineprogram = self.die.dwarfinfo.line_program_for_CU(cu)
-            filename = cu._lineprogram.header.file_entry[val-1].name.decode('utf-8', errors='ignore') if cu._lineprogram and val > 0 and val <= len(cu._lineprogram.header.file_entry) else '(N/A)'
-            return "%d: %s" % (val,  filename)
-        elif key == 'DW_AT_stmt_list':
-            return 'LNP at 0x%x' % val
-        elif isinstance(val, bytes):
-            if form in ('DW_FORM_strp', 'DW_FORM_string'):
-                return val.decode('utf-8', errors='ignore')
-            elif val == b'': # What's a good value for a blank blob?
-                return '[]'
-            elif len(val) > MAX_INLINE_BYTEARRAY_LEN:
-                return ' '.join("%02x" % b for b in val[0:MAX_INLINE_BYTEARRAY_LEN]) + ("...(%s bytes)" % (('0x%x' if self.hex else '%d') % len(val)))
-            else:
-                return ' '.join("%02x" % b for b in val) # Something like "01 ff 33 55"
-        elif isinstance(val, list): # block1 comes across as this
-            if val == []:
-                return '[]'
-            elif isinstance(val[0], int): # Assuming it's a byte array diguised as int array
-                if len(val) > MAX_INLINE_BYTEARRAY_LEN:
+            header = self.die.cu.header
+            dwarf_version = self.die.cu.header.version                               
+
+            key = attr.name
+            val = attr.value
+            form = attr.form
+            if form == 'DW_FORM_addr' and isinstance(val, int):
+                return hex(val)
+            elif form == 'DW_FORM_flag_present':
+                return ''
+            elif form in ('DW_FORM_ref0', 'DW_FORM_ref1', 'DW_FORM_ref2', 'DW_FORM_ref4', 'DW_FORM_ref8', 'DW_FORM_ref_addr'):
+                return "Ref: 0x%x" % val # There are several other reference forms in the spec
+            elif form == 'DW_FORM_flag':
+                return str(bool(val))
+            elif LocationParser.attribute_has_location(attr, self.die.cu['version']):
+                ll = self.parse_location(attr)
+                if isinstance(ll, LocationExpr):
+                    return '; '.join(self.dump_expr(ll.loc_expr))
+                else:
+                    return "Loc list: 0x%x" % attr.value
+            elif key == 'DW_AT_language':
+                return "%d %s" % (val, _DESCR_DW_LANG[val]) if val in _DESCR_DW_LANG else val
+            elif key == 'DW_AT_encoding':
+                return "%d %s" % (val, _DESCR_DW_ATE[val]) if val in _DESCR_DW_ATE else val
+            elif key == 'DW_AT_accessibility':
+                return "%d %s" % (val, _DESCR_DW_ACCESS[val]) if val in _DESCR_DW_ACCESS else val
+            elif key == 'DW_AT_inline':
+                return "%d %s" % (val, _DESCR_DW_INL[val]) if val in _DESCR_DW_INL else val
+            elif key in ('DW_AT_decl_file', 'DW_AT_call_file'):
+                cu = self.die.cu
+                if cu._lineprogram is None:
+                    cu._lineprogram = self.die.dwarfinfo.line_program_for_CU(cu)
+                filename = cu._lineprogram.header.file_entry[val-1].name.decode('utf-8', errors='ignore') if cu._lineprogram and val > 0 and val <= len(cu._lineprogram.header.file_entry) else '(N/A)'
+                return "%d: %s" % (val,  filename)
+            elif key == 'DW_AT_stmt_list':
+                return 'LNP at 0x%x' % val
+            elif key in ('DW_AT_upper_bound', 'DW_AT_lower_bound') and is_block(form):
+                return '; '.join(self.dump_expr(val))
+            elif isinstance(val, bytes):
+                if form in ('DW_FORM_strp', 'DW_FORM_string'):
+                    return val.decode('utf-8', errors='ignore')
+                elif val == b'': # What's a good value for a blank blob?
+                    return '[]'
+                elif len(val) > MAX_INLINE_BYTEARRAY_LEN:
                     return ' '.join("%02x" % b for b in val[0:MAX_INLINE_BYTEARRAY_LEN]) + ("...(%s bytes)" % (('0x%x' if self.hex else '%d') % len(val)))
                 else:
-                    return ' '.join("%02x" % b for b in val)
-            else: # List of something else
-                return str(val)
-        else:
-            return hex(val) if self.hex and isinstance(val, int) else str(val)
+                    return ' '.join("%02x" % b for b in val) # Something like "01 ff 33 55"
+            elif isinstance(val, list): # block1 comes across as this
+                if val == []:
+                    return '[]'
+                elif isinstance(val[0], int): # Assuming it's a byte array diguised as int array
+                    if len(val) > MAX_INLINE_BYTEARRAY_LEN:
+                        return ' '.join("%02x" % b for b in val[0:MAX_INLINE_BYTEARRAY_LEN]) + ("...(%s bytes)" % (('0x%x' if self.hex else '%d') % len(val)))
+                    else:
+                        return ' '.join("%02x" % b for b in val)
+                else: # List of something else
+                    return str(val)
+            else:
+                return hex(val) if self.hex and isinstance(val, int) else str(val)
+        except ELFParseError as exc:
+            from .__main__ import version
+            from .crash import report_crash
+            tb = exc.__traceback__
+            report_crash(exc, tb, version)
+            return "(parse error)"
 
     def format_form(self, form):
         return form if self.prefix or not str(form).startswith('DW_FORM_') else form[8:]
@@ -292,24 +320,24 @@ class DIETableModel(QAbstractTableModel):
             return str(val)
 
     def display_DIE(self, die):
-        rows_was = len(self.keys)
+        rows_was = len(self.keys) + self.meta_count
         self.die = die
         self.attributes = die.attributes
         self.keys = list(die.attributes.keys())
         if die.dwarfinfo.config.machine_arch != self.arch:
             self.arch = die.dwarfinfo.config.machine_arch
             self.regnamelist = _REG_NAME_MAP.get(self.arch)        
-        rows_now = len(self.keys)
+        rows_now = self.meta_count + len(self.keys)
         if rows_was < rows_now:
             self.rowsInserted.emit(QModelIndex(), rows_was, rows_now-1)
         elif rows_was > rows_now:
             self.rowsRemoved.emit(QModelIndex(), rows_now, rows_was-1)
-        self.dataChanged.emit(self.createIndex(0, 0), self.createIndex(rows_now-1, 3))
+        self.dataChanged.emit(self.createIndex(0, 0), self.createIndex(rows_now - 1, len(self.headers)-1))
 
     def set_prefix(self, prefix):
         if prefix != self.prefix:
             self.prefix = prefix
-            self.dataChanged.emit(self.createIndex(0, 2), self.createIndex(len(self.keys)-1, 3))
+            self.refresh_values()
 
     # Index is the current selected index
     # Returns the new selected index, if there was one
@@ -337,16 +365,19 @@ class DIETableModel(QAbstractTableModel):
                     new_index = QModelIndex() # Select none
         return new_index
 
+    # Force a reload of values on the whole table without row/column count change
+    def refresh_values(self):
+        self.dataChanged.emit(self.createIndex(0, 0), self.createIndex(self.meta_count + len(self.keys)-1, len(self.headers)-1))
 
     def set_hex(self, hex):
         if hex != self.hex:
             self.hex = hex
-            self.dataChanged.emit(self.createIndex(0, 0), self.createIndex(len(self.keys)-1, 0))
+            self.refresh_values()
 
     def set_regnames(self, regnames):
         if regnames != self.regnames:
             self.regnames = regnames
-            self.dataChanged.emit(self.createIndex(0, 0), self.createIndex(len(self.keys)-1, 0))
+            self.refresh_values()
 
     # Returns a table model for the attribute details table
     # For attributes that refer to larger data structures (ranges, locations), makes sense to spell it out into a table
@@ -366,9 +397,17 @@ class DIETableModel(QAbstractTableModel):
                     return None
                 ranges = di._ranges.get_range_list_at_offset(attr.value)
                 # TODO: handle base addresses. Never seen those so far...
-                cu_base = get_cu_base(self.die)
-                return GenericTableModel(("Start offset", "End offset"),
-                    ((hex(cu_base + r.begin_offset), hex(cu_base + r.end_offset)) for r in ranges))
+                try:
+                    top_die = self.die.cu.get_top_DIE() 
+                    cu_base = get_cu_base(self.die)
+                    return GenericTableModel(("Start offset", "End offset"),
+                        ((hex(cu_base + r.begin_offset), hex(cu_base + r.end_offset)) for r in ranges))
+                except NoBaseError as exc:
+                    from .__main__ import version
+                    from .crash import report_crash
+                    tb = exc.__traceback__
+                    report_crash(exc, tb, version)
+                    return None
             elif LocationParser.attribute_has_location(attr, self.die.cu['version']):
                 # Expression is a list of ints
                 ll = self.parse_location(attr)
@@ -416,6 +455,8 @@ class DIETableModel(QAbstractTableModel):
                 # TODO: low level flavor with extra details
                 # TODO: commands vs states
                 return GenericTableModel(('Address', 'File', 'Line', 'Stmt', 'Basic block', 'End seq', 'End prologue', 'Begin epilogue'), states)
+            elif key in ('DW_AT_upper_bound', 'DW_AT_lower_bound') and is_block(form):
+                return GenericTableModel(("Command",), [(o,) for o in self.dump_expr(attr.value)])
             elif is_long_blob(attr):
                 val = attr.value
                 def format_line(off):
@@ -428,12 +469,18 @@ class DIETableModel(QAbstractTableModel):
     # Returns (cu, die_offset) or None if not a navigable
     def ref_target(self, index):
         try:  # Any chance for "not found"? Probably bug #1450
-            attr = self.attributes[self.keys[index.row() - self.meta_count]]
+            irow = index.row()
+            meta_count = self.meta_count
+            attr_index = irow - meta_count
+            self_keys_len = len(self.keys)
+            self_die_attr_keys_len = len(self.die.attributes.keys())
+            attr_name = self.keys[attr_index]
+            attr = self.attributes[attr_name]
             val = attr.value
             form = attr.form
-            if attr.form in ('DW_FORM_ref1', 'DW_FORM_ref2', 'DW_FORM_ref4', 'DW_FORM_ref8'):
+            if form in ('DW_FORM_ref1', 'DW_FORM_ref2', 'DW_FORM_ref4', 'DW_FORM_ref8'):
                 return (self.die.cu, attr.value + self.die.cu.cu_offset)
-            elif attr.form in ('DW_FORM_ref_addr', 'DW_FORM_ref'):
+            elif form in ('DW_FORM_ref_addr', 'DW_FORM_ref'):
                 cualen = len(self.die.cu.dwarfinfo._unsorted_CUs)
                 i = bisect_right(self.die.cu.dwarfinfo._CU_offsets, val) - 1
                 cu = self.die.cu.dwarfinfo._unsorted_CUs[i]
